# Schema for Recraft V3 Text-to-Image and Image-to-Image generation
from typing import Literal
from pydantic import BaseModel, Field, HttpUrl

class TextToImageInput(BaseModel):
    prompt: str = Field(
        ...,
        json_schema_extra={"x-sr-order": 201},
        max_length=1000,
        description="The prompt for the image generation"
    )
    aspect_ratio: Literal[
        "1:1", "4:3", "3:4", "16:9", "9:16"
    ] = Field(
        "1:1",
        json_schema_extra={"x-sr-order": 401},
        description="Aspect ratio of the image"
    )
    style: Literal[
        "any", "realistic_image", "digital_illustration", "vector_illustration",
        "realistic_image/b_and_w", "realistic_image/hard_flash", "realistic_image/hdr",
        "realistic_image/natural_light", "realistic_image/studio_portrait",
        "realistic_image/enterprise", "realistic_image/motion_blur",
        "realistic_image/evening_light", "realistic_image/faded_nostalgia",
        "realistic_image/forest_life", "realistic_image/mystic_naturalism",
        "realistic_image/natural_tones", "realistic_image/organic_calm",
        "realistic_image/real_life_glow", "realistic_image/retro_realism",
        "realistic_image/retro_snapshot", "realistic_image/urban_drama",
        "realistic_image/village_realism", "realistic_image/warm_folk",
        "digital_illustration/pixel_art", "digital_illustration/hand_drawn",
        "digital_illustration/grain", "digital_illustration/infantile_sketch",
        "digital_illustration/2d_art_poster", "digital_illustration/handmade_3d",
        "digital_illustration/hand_drawn_outline", "digital_illustration/engraving_color",
        "digital_illustration/2d_art_poster_2", "digital_illustration/antiquarian",
        "digital_illustration/bold_fantasy", "digital_illustration/child_book",
        "digital_illustration/child_books", "digital_illustration/cover",
        "digital_illustration/crosshatch", "digital_illustration/digital_engraving",
        "digital_illustration/expressionism", "digital_illustration/freehand_details",
        "digital_illustration/grain_20", "digital_illustration/graphic_intensity",
        "digital_illustration/hard_comics", "digital_illustration/long_shadow",
        "digital_illustration/modern_folk", "digital_illustration/multicolor",
        "digital_illustration/neon_calm", "digital_illustration/noir",
        "digital_illustration/nostalgic_pastel", "digital_illustration/outline_details",
        "digital_illustration/pastel_gradient", "digital_illustration/pastel_sketch",
        "digital_illustration/pop_art", "digital_illustration/pop_renaissance",
        "digital_illustration/street_art", "digital_illustration/tablet_sketch",
        "digital_illustration/urban_glow", "digital_illustration/urban_sketching",
        "digital_illustration/vanilla_dreams", "digital_illustration/young_adult_book",
        "digital_illustration/young_adult_book_2", "vector_illustration/bold_stroke",
        "vector_illustration/chemistry", "vector_illustration/colored_stencil",
        "vector_illustration/contour_pop_art", "vector_illustration/cosmics",
        "vector_illustration/cutout", "vector_illustration/depressive",
        "vector_illustration/editorial", "vector_illustration/emotional_flat",
        "vector_illustration/infographical", "vector_illustration/marker_outline",
        "vector_illustration/mosaic", "vector_illustration/naivector",
        "vector_illustration/roundish_flat", "vector_illustration/segmented_colors",
        "vector_illustration/sharp_contrast", "vector_illustration/thin",
        "vector_illustration/vector_photo", "vector_illustration/vivid_shapes",
        "vector_illustration/engraving", "vector_illustration/line_art",
        "vector_illustration/line_circuit", "vector_illustration/linocut"
    ] = Field(
        "realistic_image",
        json_schema_extra={"x-sr-order": 402},
        description="The style of the generated images"
    )
    style_id: str = Field(
        None,
        json_schema_extra={"x-sr-order": 403},
        description="The ID of the custom style reference (optional)"
    )


class ImageToImageInput(BaseModel):
    prompt: str = Field(
        ...,
        json_schema_extra={"x-sr-order": 201},
        max_length=1000,
        description="A text description of areas to change"
    )
    image: HttpUrl | str = Field(
        ...,
        json_schema_extra={"x-sr-order": 301},
        description="URL of the image to modify"
    )
    image_strength: float = Field(
        0.5,
        ge=0.0,
        le=1.0,
        multiple_of=0.01,
        json_schema_extra={"x-sr-order": 302},
        description="Defines the difference with the original image, should lie in [0, 1], where 0 means almost identical, and 1 means miserable similarity"
    )
    style: Literal[
        "any", "realistic_image", "digital_illustration", "vector_illustration",
        "realistic_image/b_and_w", "realistic_image/hard_flash", "realistic_image/hdr",
        "realistic_image/natural_light", "realistic_image/studio_portrait",
        "realistic_image/enterprise", "realistic_image/motion_blur",
        "realistic_image/evening_light", "realistic_image/faded_nostalgia",
        "realistic_image/forest_life", "realistic_image/mystic_naturalism",
        "realistic_image/natural_tones", "realistic_image/organic_calm",
        "realistic_image/real_life_glow", "realistic_image/retro_realism",
        "realistic_image/retro_snapshot", "realistic_image/urban_drama",
        "realistic_image/village_realism", "realistic_image/warm_folk",
        "digital_illustration/pixel_art", "digital_illustration/hand_drawn",
        "digital_illustration/grain", "digital_illustration/infantile_sketch",
        "digital_illustration/2d_art_poster", "digital_illustration/handmade_3d",
        "digital_illustration/hand_drawn_outline", "digital_illustration/engraving_color",
        "digital_illustration/2d_art_poster_2", "digital_illustration/antiquarian",
        "digital_illustration/bold_fantasy", "digital_illustration/child_book",
        "digital_illustration/child_books", "digital_illustration/cover",
        "digital_illustration/crosshatch", "digital_illustration/digital_engraving",
        "digital_illustration/expressionism", "digital_illustration/freehand_details",
        "digital_illustration/grain_20", "digital_illustration/graphic_intensity",
        "digital_illustration/hard_comics", "digital_illustration/long_shadow",
        "digital_illustration/modern_folk", "digital_illustration/multicolor",
        "digital_illustration/neon_calm", "digital_illustration/noir",
        "digital_illustration/nostalgic_pastel", "digital_illustration/outline_details",
        "digital_illustration/pastel_gradient", "digital_illustration/pastel_sketch",
        "digital_illustration/pop_art", "digital_illustration/pop_renaissance",
        "digital_illustration/street_art", "digital_illustration/tablet_sketch",
        "digital_illustration/urban_glow", "digital_illustration/urban_sketching",
        "digital_illustration/vanilla_dreams", "digital_illustration/young_adult_book",
        "digital_illustration/young_adult_book_2", "vector_illustration/bold_stroke",
        "vector_illustration/chemistry", "vector_illustration/colored_stencil",
        "vector_illustration/contour_pop_art", "vector_illustration/cosmics",
        "vector_illustration/cutout", "vector_illustration/depressive",
        "vector_illustration/editorial", "vector_illustration/emotional_flat",
        "vector_illustration/infographical", "vector_illustration/marker_outline",
        "vector_illustration/mosaic", "vector_illustration/naivector",
        "vector_illustration/roundish_flat", "vector_illustration/segmented_colors",
        "vector_illustration/sharp_contrast", "vector_illustration/thin",
        "vector_illustration/vector_photo", "vector_illustration/vivid_shapes",
        "vector_illustration/engraving", "vector_illustration/line_art",
        "vector_illustration/line_circuit", "vector_illustration/linocut"
    ] = Field(
        "realistic_image",
        json_schema_extra={"x-sr-order": 402},
        description="The style of the generated images"
    )
    style_id: str = Field(
        None,
        json_schema_extra={"x-sr-order": 403},
        description="The ID of the custom style reference (optional)"
    ) 